@@ -8,16 +8,17 @@ function(libname, pkgname)
  }
 
    # Set the error handlers to our local ones.
- .C("RSXML_setErrorHandlers")
+ .C("RSXML_setErrorHandlers", PACKAGE = "XML")
 }
 
 .onLoad =
 function(libname, pkgname)
 {
+
   # Added by Uwe Ligges.
  if(.Platform$OS.type == "windows"){
      temp <- Sys.getenv("PATH")
-     Sys.setenv("PATH" = paste(normalizePath(file.path(libname, pkgname, "libs")), 
+     Sys.setenv("PATH" = paste(utils::normalizePath(file.path(libname, pkgname, "libs")), 
                                file.path(Sys.getenv("R_HOME"), "modules", fsep="\\"), temp, sep=";"))
      on.exit(Sys.setenv(PATH = temp))
  }     
@@ -28,7 +29,7 @@ function(libname, pkgname)
 #   .InitSAXMethods()
  }
    # Set the error handlers to our local ones.
- .C("RSXML_setErrorHandlers")
+ .C("RSXML_setErrorHandlers", PACKAGE = "XML")
 }
 
 .onUnload <- function (libpath)
@@ -38,18 +39,20 @@ function(libname, pkgname)
 
 
 
+if(FALSE) {
 .Call =
 function(name, ...)
 {
   base::.Call(name, ..., PACKAGE = "XML")
 }
 
+
 .C =
 function(name, ...)
 {
   base::.C(name, ..., PACKAGE = "XML")
 }
-
+}
 
 #
 #  Copyright (c) 1998, 1999 The Omega Project for Statistical Computing.
