@@ -340,7 +340,7 @@ R_processBranch(RS_XMLParserData * rinfo,
 
     node = xmlNewNode(NULL, localname);
     if(attributes) {
-        xmlChar ** p = attributes;
+        const xmlChar ** p = attributes;
         int i;
 	if(sax1) {
 	  for(i = 0; *p ; i += 2, p += 2) 
@@ -371,7 +371,9 @@ R_xmlFreeNode(SEXP node)
   p = R_ExternalPtrAddr(node);
   if(p) {
       xmlFreeNode(p);
+#ifdef R_XML_DEBUG
 fprintf(stderr, "Freeing XML node from a branch\n");
+#endif
   }
   R_SetExternalPtrAddr(node, NULL_USER_OBJECT);
 }
